@@ -89,6 +89,8 @@ flatpak_builtin_enter (int           argc,
   g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
 
   rest_argc = 0;
+  rest_argv_start = 0;
+
   for (i = 1; i < argc; i++)
     {
       /* The non-option is the command, take it out of the arguments */
@@ -110,6 +112,9 @@ flatpak_builtin_enter (int           argc,
       return FALSE;
     }
 
+  /* If this wasn't true, rest_argc would still be 0 */
+  g_assert (rest_argv_start >= 1);
+
   pid_s = argv[rest_argv_start];
   pid = atoi (pid_s);
 
@@ -121,7 +126,7 @@ flatpak_builtin_enter (int           argc,
       FlatpakInstance *instance = (FlatpakInstance *) g_ptr_array_index (instances, j);
 
       if (pid == flatpak_instance_get_pid (instance) ||
-          strcmp (pid_s, flatpak_instance_get_app (instance)) == 0 ||
+          g_strcmp0 (pid_s, flatpak_instance_get_app (instance)) == 0 ||
           strcmp (pid_s, flatpak_instance_get_id (instance)) == 0)
         {
           pid = flatpak_instance_get_child_pid (instance);
@@ -271,10 +276,13 @@ flatpak_builtin_enter (int           argc,
 
   session_bus_path = g_strdup_printf ("/run/user/%d/bus", uid);
   if (g_file_test (session_bus_path, G_FILE_TEST_EXISTS))
-    g_ptr_array_add (envp_array, g_strdup_printf ("DBUS_SESSION_BUS_ADDRESS=unix:%s", session_bus_path));
+    g_ptr_array_add (envp_array, g_strdup_printf ("DBUS_SESSION_BUS_ADDRESS=unix:path=%s", session_bus_path));
 
   if (g_file_test ("/run/dbus/system_bus_socket", G_FILE_TEST_EXISTS))
-    g_ptr_array_add (envp_array, g_strdup ("DBUS_SYSTEM_BUS_ADDRESS=unix:/run/dbus/system_bus_socket"));
+    g_ptr_array_add (envp_array, g_strdup ("DBUS_SYSTEM_BUS_ADDRESS=unix:path=/run/dbus/system_bus_socket"));
+
+  if (getenv ("TERM"))
+    g_ptr_array_add (envp_array, g_strdup_printf ("TERM=%s", getenv ("TERM")));
 
   g_ptr_array_add (envp_array, NULL);
 
@@ -283,6 +291,7 @@ flatpak_builtin_enter (int           argc,
     g_ptr_array_add (argv_array, g_strdup (argv[rest_argv_start + i]));
   g_ptr_array_add (argv_array, NULL);
 
+  signal (SIGINT, SIG_IGN);
   if (!g_spawn_sync (NULL, (char **) argv_array->pdata, (char **) envp_array->pdata,
                      G_SPAWN_SEARCH_PATH_FROM_ENVP | G_SPAWN_CHILD_INHERITS_STDIN,
                      NULL, NULL,
@@ -315,7 +324,11 @@ flatpak_complete_enter (FlatpakCompletion *completion)
       for (i = 0; i < instances->len; i++)
         {
           FlatpakInstance *instance = (FlatpakInstance *) g_ptr_array_index (instances, i);
-          flatpak_complete_word (completion, "%s ", flatpak_instance_get_app (instance));
+
+          const char *app_name = flatpak_instance_get_app (instance);
+          if (app_name)
+            flatpak_complete_word (completion, "%s ", app_name);
+
           flatpak_complete_word (completion, "%s ", flatpak_instance_get_id (instance));
         }
       break;
