@@ -48,11 +48,11 @@ flatpak_builtin_make_current_app (int argc, char **argv, GCancellable *cancellab
   g_autoptr(GFile) deploy_base = NULL;
   const char *pref;
   const char *default_branch = NULL;
-  g_autofree char *ref = NULL;
   g_auto(GLnxLockFile) lock = { 0, };
   g_autofree char *id = NULL;
   g_autofree char *arch = NULL;
   g_autofree char *branch = NULL;
+  g_autoptr(FlatpakDecomposed) ref = NULL;
   FlatpakKinds kinds;
 
   context = g_option_context_new (_("APP BRANCH - Make branch of application current"));
@@ -84,7 +84,7 @@ flatpak_builtin_make_current_app (int argc, char **argv, GCancellable *cancellab
     return usage_error (context, _("BRANCH must be specified"), error);
 
   ref = flatpak_dir_find_installed_ref (dir, id, branch, arch, FLATPAK_KINDS_APP,
-                                        NULL, error);
+                                        error);
   if (ref == NULL)
     return FALSE;
 
@@ -117,12 +117,12 @@ flatpak_complete_make_current_app (FlatpakCompletion *completion)
   g_autoptr(GPtrArray) dirs = NULL;
   FlatpakDir *dir;
   g_autoptr(GError) error = NULL;
-  g_auto(GStrv) refs = NULL;
-  int i;
+  g_autoptr(GPtrArray) refs = NULL;
 
   context = g_option_context_new ("");
   if (!flatpak_option_context_parse (context, options, &completion->argc, &completion->argv,
-                                     FLATPAK_BUILTIN_FLAG_ONE_DIR, &dirs, NULL, NULL))
+                                     FLATPAK_BUILTIN_FLAG_ONE_DIR | FLATPAK_BUILTIN_FLAG_OPTIONAL_REPO,
+                                     &dirs, NULL, NULL))
     return FALSE;
 
   dir = g_ptr_array_index (dirs, 0);
@@ -136,28 +136,23 @@ flatpak_complete_make_current_app (FlatpakCompletion *completion)
       flatpak_complete_options (completion, user_entries);
 
       refs = flatpak_dir_find_installed_refs (dir, NULL, NULL, opt_arch,
-                                              FLATPAK_KINDS_APP, &error);
+                                              FLATPAK_KINDS_APP,
+                                              FIND_MATCHING_REFS_FLAGS_NONE,
+                                              &error);
       if (refs == NULL)
         flatpak_completion_debug ("find installed refs error: %s", error->message);
-      for (i = 0; refs != NULL && refs[i] != NULL; i++)
-        {
-          g_auto(GStrv) parts = flatpak_decompose_ref (refs[i], NULL);
-          if (parts)
-            flatpak_complete_word (completion, "%s ", parts[1]);
-        }
+      flatpak_complete_ref_id (completion, refs);
       break;
 
     case 2: /* Branch */
       refs = flatpak_dir_find_installed_refs (dir, completion->argv[1], NULL, opt_arch,
-                                              FLATPAK_KINDS_APP, &error);
+                                              FLATPAK_KINDS_APP,
+                                              FIND_MATCHING_REFS_FLAGS_NONE,
+                                              &error);
       if (refs == NULL)
         flatpak_completion_debug ("find installed refs error: %s", error->message);
-      for (i = 0; refs != NULL && refs[i] != NULL; i++)
-        {
-          g_auto(GStrv) parts = flatpak_decompose_ref (refs[i], NULL);
-          if (parts)
-            flatpak_complete_word (completion, "%s ", parts[3]);
-        }
+
+      flatpak_complete_ref_branch (completion, refs);
       break;
 
     default:
