@@ -280,7 +280,7 @@ flatpak_builtin_remote_info (int argc, char **argv, GCancellable *cancellable, G
       print_aligned (len, _("ID:"), id);
       print_aligned (len, _("Ref:"), flatpak_decomposed_get_ref (ref));
       print_aligned_take (len, _("Arch:"), flatpak_decomposed_dup_arch (ref));
-      print_aligned (len, _("Branch:"), flatpak_decomposed_dup_arch (ref));
+      print_aligned_take (len, _("Branch:"), flatpak_decomposed_dup_branch (ref));
       if (version != NULL)
         print_aligned (len, _("Version:"), version);
       if (license != NULL)
@@ -441,10 +441,7 @@ flatpak_builtin_remote_info (int argc, char **argv, GCancellable *cancellable, G
 
           if (opt_show_metadata)
             {
-              if (xa_metadata != NULL)
-                flatpak_print_escaped_string (xa_metadata,
-                                              FLATPAK_ESCAPE_ALLOW_NEWLINES
-                                              | FLATPAK_ESCAPE_DO_NOT_QUOTE);
+              g_print ("%s", xa_metadata ? xa_metadata : "");
               if (xa_metadata == NULL || !g_str_has_suffix (xa_metadata, "\n"))
                 g_print ("\n");
             }
