@@ -50,22 +50,29 @@ static Column all_columns[] = {
   { "title",      N_("Title"),         N_("Show the title"),         0, FLATPAK_ELLIPSIZE_MODE_NONE, 1, 0 },
   { "url",        N_("URL"),           N_("Show the URL"),           0, FLATPAK_ELLIPSIZE_MODE_NONE, 1, 0 },
   { "collection", N_("Collection ID"), N_("Show the collection ID"), 0, FLATPAK_ELLIPSIZE_MODE_NONE, 1, 0 },
+  { "subset",     N_("Subset"),        N_("Show the subset"),        0, FLATPAK_ELLIPSIZE_MODE_NONE, 1, 0 },
+  { "filter",     N_("Filter"),        N_("Show filter file"),       0, FLATPAK_ELLIPSIZE_MODE_NONE, 1, 0 },
   { "priority",   N_("Priority"),      N_("Show the priority"),      0, FLATPAK_ELLIPSIZE_MODE_NONE, 1, 0 },
   { "options",    N_("Options"),       N_("Show options"),           0, FLATPAK_ELLIPSIZE_MODE_NONE, 1, 1 },
+  { "comment",    N_("Comment"),       N_("Show comment"),           0, FLATPAK_ELLIPSIZE_MODE_END,  1, 0 },
+  { "description", N_("Description"),  N_("Show description"),       0, FLATPAK_ELLIPSIZE_MODE_END,  1, 0 },
+  { "homepage",   N_("Homepage"),      N_("Show homepage"),          0, FLATPAK_ELLIPSIZE_MODE_NONE,  1, 0 },
+  { "icon",       N_("Icon"),          N_("Show icon"),              0, FLATPAK_ELLIPSIZE_MODE_NONE,  1, 0 },
   { NULL }
 };
 
 static gboolean
 list_remotes (GPtrArray *dirs, Column *columns, GCancellable *cancellable, GError **error)
 {
-  FlatpakTablePrinter *printer;
+  g_autoptr(FlatpakTablePrinter) printer = NULL;
   int i, j, k;
 
   if (columns[0].name == NULL)
     return TRUE;
 
   printer = flatpak_table_printer_new ();
-  flatpak_table_printer_set_columns (printer, columns);
+  flatpak_table_printer_set_columns (printer, columns,
+                                     opt_cols == NULL && !opt_show_details);
 
   for (j = 0; j < dirs->len; j++)
     {
@@ -97,6 +104,54 @@ list_remotes (GPtrArray *dirs, Column *columns, GCancellable *cancellable, GErro
                   else
                     flatpak_table_printer_add_column (printer, "-");
                 }
+              else if (strcmp (columns[k].name, "subset") == 0)
+                {
+                  g_autofree char *subset = flatpak_dir_get_remote_subset (dir, remote_name);
+                  if (subset)
+                    flatpak_table_printer_add_column (printer, subset);
+                  else
+                    flatpak_table_printer_add_column (printer, "-");
+                }
+              else if (strcmp (columns[k].name, "comment") == 0)
+                {
+                  g_autofree char *comment = flatpak_dir_get_remote_comment (dir, remote_name);
+                  if (comment)
+                    flatpak_table_printer_add_column (printer, comment);
+                  else
+                    flatpak_table_printer_add_column (printer, "-");
+                }
+              else if (strcmp (columns[k].name, "description") == 0)
+                {
+                  g_autofree char *description = flatpak_dir_get_remote_description (dir, remote_name);
+                  if (description)
+                    flatpak_table_printer_add_column (printer, description);
+                  else
+                    flatpak_table_printer_add_column (printer, "-");
+                }
+              else if (strcmp (columns[k].name, "filter") == 0)
+                {
+                  g_autofree char *filter = flatpak_dir_get_remote_filter (dir, remote_name);
+                  if (filter)
+                    flatpak_table_printer_add_column (printer, filter);
+                  else
+                    flatpak_table_printer_add_column (printer, "-");
+                }
+              else if (strcmp (columns[k].name, "homepage") == 0)
+                {
+                  g_autofree char *homepage = flatpak_dir_get_remote_homepage (dir, remote_name);
+                  if (homepage)
+                    flatpak_table_printer_add_column (printer, homepage);
+                  else
+                    flatpak_table_printer_add_column (printer, "-");
+                }
+              else if (strcmp (columns[k].name, "icon") == 0)
+                {
+                  g_autofree char *icon = flatpak_dir_get_remote_icon (dir, remote_name);
+                  if (icon)
+                    flatpak_table_printer_add_column (printer, icon);
+                  else
+                    flatpak_table_printer_add_column (printer, "-");
+                }
               else if (strcmp (columns[k].name, "url") == 0)
                 {
                   g_autofree char *remote_url = NULL;
@@ -123,6 +178,7 @@ list_remotes (GPtrArray *dirs, Column *columns, GCancellable *cancellable, GErro
               else if (strcmp (columns[k].name, "options") == 0)
                 {
                   gboolean gpg_verify = TRUE;
+                  g_autofree char *filter = flatpak_dir_get_remote_filter (dir, remote_name);
 
                   flatpak_table_printer_add_column (printer, ""); /* Options */
 
@@ -141,10 +197,15 @@ list_remotes (GPtrArray *dirs, Column *columns, GCancellable *cancellable, GErro
                   if (flatpak_dir_get_remote_noenumerate (dir, remote_name))
                     flatpak_table_printer_append_with_comma (printer, "no-enumerate");
 
-                  ostree_repo_remote_get_gpg_verify (flatpak_dir_get_repo (dir), remote_name,
-                                                     &gpg_verify, NULL);
+                  if (!ostree_repo_remote_get_gpg_verify (flatpak_dir_get_repo (dir), remote_name,
+                                                          &gpg_verify, error))
+                      return FALSE; /* shouldn't happen unless repo config is modified out-of-band */
+
                   if (!gpg_verify)
                     flatpak_table_printer_append_with_comma (printer, "no-gpg-verify");
+
+                  if (filter != NULL && *filter != 0)
+                    flatpak_table_printer_append_with_comma (printer, "filtered");
                 }
             }
 
@@ -153,7 +214,6 @@ list_remotes (GPtrArray *dirs, Column *columns, GCancellable *cancellable, GErro
     }
 
   flatpak_table_printer_print (printer);
-  flatpak_table_printer_free (printer);
 
   return TRUE;
 }
@@ -181,7 +241,7 @@ flatpak_builtin_remote_list (int argc, char **argv, GCancellable *cancellable, G
   columns = handle_column_args (all_columns, opt_show_details, opt_cols, error);
   if (columns == NULL)
     return FALSE;
-  
+
   return list_remotes (dirs, columns, cancellable, error);
 }
 
