@@ -460,7 +460,9 @@ flatpak_option_context_parse (GOptionContext     *context,
         {
           FlatpakDir *dir = g_ptr_array_index (dirs, i);
 
-          if (flags & FLATPAK_BUILTIN_FLAG_OPTIONAL_REPO)
+          if (flags & (FLATPAK_BUILTIN_FLAG_OPTIONAL_REPO |
+                       FLATPAK_BUILTIN_FLAG_ALL_DIRS |
+                       FLATPAK_BUILTIN_FLAG_STANDARD_DIRS))
             {
               if (!flatpak_dir_maybe_ensure_repo (dir, cancellable, error))
                 return FALSE;
@@ -817,10 +819,11 @@ flatpak_run (int      argc,
     check_environment ();
 
   /* Don't talk to dbus in enter, as it must be thread-free to setns, also
-     skip run/build for performance reasons (no need to connect to dbus). */
+     skip run/build/history for performance reasons (no need to connect to dbus). */
   if (g_strcmp0 (command->name, "enter") != 0 &&
       g_strcmp0 (command->name, "run") != 0 &&
-      g_strcmp0 (command->name, "build") != 0)
+      g_strcmp0 (command->name, "build") != 0 &&
+      g_strcmp0 (command->name, "history") != 0)
     polkit_agent = install_polkit_agent ();
 
   /* g_vfs_get_default can spawn threads */
@@ -876,7 +879,7 @@ complete (int    argc,
       FlatpakCommand *c = commands;
       while (c->name)
         {
-          if (c->fn != NULL)
+          if (c->fn != NULL && !c->deprecated)
             flatpak_complete_word (completion, "%s ", c->name);
           c++;
         }
