@@ -69,6 +69,7 @@ typedef struct
   gboolean             expand;
   FlatpakEllipsizeMode ellipsize;
   gboolean             skip_unique;
+  char                *skip_unique_str;
   gboolean             skip;
 } TableColumn;
 
@@ -78,6 +79,7 @@ free_column (gpointer data)
   TableColumn *column = data;
 
   g_free (column->title);
+  g_free (column->skip_unique_str);
   g_free (column);
 }
 
@@ -272,12 +274,9 @@ flatpak_table_printer_append_with_comma_printf (FlatpakTablePrinter *printer,
   va_list var_args;
   g_autofree char *s = NULL;
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wformat-nonliteral"
   va_start (var_args, format);
   s = g_strdup_vprintf (format, var_args);
   va_end (var_args);
-#pragma GCC diagnostic pop
 
   flatpak_table_printer_append_with_comma (printer, s);
 }
@@ -391,7 +390,8 @@ string_add_spaces (GString *str, int count)
 static gboolean
 column_is_unique (FlatpakTablePrinter *printer, int col)
 {
-  char *first_row = NULL;
+  TableColumn *column = get_table_column (printer, col);
+  char *first_row = column->skip_unique_str;
   int i;
 
   for (i = 0; i < printer->rows->len; i++)
@@ -402,7 +402,7 @@ column_is_unique (FlatpakTablePrinter *printer, int col)
 
       Cell *cell = g_ptr_array_index (row->cells, col);
 
-      if (i == 0)
+      if (i == 0 && first_row == NULL)
         first_row = cell->text;
       else
         {
@@ -853,6 +853,7 @@ flatpak_table_printer_set_column_ellipsize (FlatpakTablePrinter *printer,
   col->ellipsize = mode;
 }
 
+/* Specifies that the column should be skipped if all values are the same */
 void
 flatpak_table_printer_set_column_skip_unique (FlatpakTablePrinter *printer,
                                               int                  column,
@@ -862,3 +863,20 @@ flatpak_table_printer_set_column_skip_unique (FlatpakTablePrinter *printer,
 
   col->skip_unique = skip_unique;
 }
+
+/* This modifies set_column_skip_unique to also require that the
+ * unique value of the column must be this particular string. Useful if you
+ * want to e.g. skip the arch list if everything is for the primary arch, but
+ * not if everything is for a non-standard arch.
+ */
+void
+flatpak_table_printer_set_column_skip_unique_string (FlatpakTablePrinter *printer,
+                                                     int                  column,
+                                                     const char          *str)
+{
+  TableColumn *col = get_table_column (printer, column);
+
+  g_assert (col->skip_unique_str == NULL);
+
+  col->skip_unique_str = g_strdup (str);
+}
