@@ -29,6 +29,7 @@
 #include <gio/gio.h>
 #include <gio/gunixfdlist.h>
 #include "flatpak-dbus-generated.h"
+#include "flatpak-session-helper.h"
 #include "flatpak-utils-base-private.h"
 
 typedef enum {
@@ -107,8 +108,8 @@ child_watch_died (GPid     pid,
   signal_variant = g_variant_ref_sink (g_variant_new ("(uu)", pid, status));
   g_dbus_connection_emit_signal (session_bus,
                                  pid_data->client,
-                                 "/org/freedesktop/Flatpak/Development",
-                                 "org.freedesktop.Flatpak.Development",
+                                 FLATPAK_SESSION_HELPER_PATH_DEVELOPMENT,
+                                 FLATPAK_SESSION_HELPER_INTERFACE_DEVELOPMENT,
                                  "HostCommandExited",
                                  signal_variant,
                                  NULL);
@@ -469,7 +470,7 @@ on_bus_acquired (GDBusConnection *connection,
 
   if (!g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (helper),
                                          connection,
-                                         "/org/freedesktop/Flatpak/SessionHelper",
+                                         FLATPAK_SESSION_HELPER_PATH,
                                          &error))
     {
       g_warning ("error: %s", error->message);
@@ -483,7 +484,7 @@ on_bus_acquired (GDBusConnection *connection,
 
   if (!g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (devel),
                                          connection,
-                                         "/org/freedesktop/Flatpak/Development",
+                                         FLATPAK_SESSION_HELPER_PATH_DEVELOPMENT,
                                          &error))
     {
       g_warning ("error: %s", error->message);
@@ -777,7 +778,11 @@ main (int    argc,
     { "version", 0, 0, G_OPTION_ARG_NONE, &show_version, "Show program version.", NULL},
     { NULL }
   };
-  g_autoptr(MonitorData) m_resolv_conf = NULL, m_host_conf = NULL, m_hosts = NULL, m_localtime = NULL;
+  g_autoptr(MonitorData) m_resolv_conf = NULL,
+                         m_host_conf = NULL,
+                         m_hosts = NULL,
+                         m_gai_conf = NULL,
+                         m_localtime = NULL;
   struct sigaction action;
 
   atexit (do_atexit);
@@ -856,6 +861,7 @@ main (int    argc,
   m_resolv_conf = setup_file_monitor ("/etc/resolv.conf");
   m_host_conf   = setup_file_monitor ("/etc/host.conf");
   m_hosts       = setup_file_monitor ("/etc/hosts");
+  m_gai_conf    = setup_file_monitor ("/etc/gai.conf");
   m_localtime   = setup_file_monitor ("/etc/localtime");
 
   flags = G_BUS_NAME_OWNER_FLAGS_ALLOW_REPLACEMENT;
@@ -863,7 +869,7 @@ main (int    argc,
     flags |= G_BUS_NAME_OWNER_FLAGS_REPLACE;
 
   owner_id = g_bus_own_name (G_BUS_TYPE_SESSION,
-                             "org.freedesktop.Flatpak",
+                             FLATPAK_SESSION_HELPER_BUS_NAME,
                              flags,
                              on_bus_acquired,
                              on_name_acquired,
